#ifndef BITTORRENT_HH
#define BITTORRENT_HH

#include <string>
#include <map>
#include <list>
#include <vector>

#define MAXSIZE_BT_FILE (5*1024*1024)

#define BT_DICTIONARY 0
#define BT_STRING     1
#define BT_INTEGER    2
#define BT_LIST       3

typedef std::list< std::pair< std::string, int > > BT_FILES;
typedef std::map< std::string, void* >             dictionary_t;
typedef std::list< class elm* >                    list_t;

class elm {
public:
	elm() { val = NULL; type = -1; };
	~elm() {
		switch( type ) {
			case BT_INTEGER:
				if( val ) {
					delete (int*) (val);
				}
				break;
			case BT_STRING:
				if( val ) {
					delete (std::string*) (val);
				}
				break;
			case BT_LIST:
				for( list_t::iterator it = ( (list_t*) val )->begin(); it != ( (list_t*) val )->end(); ++it ) {
					if( *it ) {
						delete *it;
					}
				}
				if( val ) {
						delete (list_t*) (val);
				}
				break;
			case BT_DICTIONARY:
				if( val ) {
					for( dictionary_t::iterator it = ( (dictionary_t*) val )->begin(); 
							it != ( (dictionary_t*) val )->end(); ++it ) {
						elm* e = (elm*) it->second;
						delete e;
					}
				}
				if( val ) {
					delete (dictionary_t*) (val);
				}
				break;
		}
	}
	int   type;
	void* val;
};

class BitTorrent
{
private:
	BitTorrent();
public:
	~BitTorrent();
	static BitTorrent* decode( const char* buf, int len );
	static BitTorrent* BitTorrent::bt_decode_file( const char* filename );
	void setAnnounce( const std::string& announce );
	const std::string& getAnnounce() const;
	void setSingleFile( bool s );
	bool singleFile() const;
	void addFile( const std::string& fname, int len );
	void setDirectory( const std::string& dirname );
	const std::string& getDirectory() const;
	void setPieceLength( int l );
	int getPieceLength() const;
	void setPieces( const std::string& p );
	const std::string& getPiece( int n ) const;
	int pieces() const;
	int size() const;
	void addSize( int len );
	bool getPieces( const std::string& file, int&, int&, int& ) const;
	int getLength( const std::string& file ) const;
	//! Decodes the bencoded buffer.
	static elm* get_dict_value( elm* e, const std::string& key );
	static elm* parse( const char* buf, int len );
protected:
	static elm* parse( char** buf, int* pos, int len );
	static std::string* get_string( char** buf, int* pos, int l );
	static dictionary_t* get_dictionary( char** buf, int* pos, int len );
	static list_t* get_list( char** buf, int* pos, int len );
	static int* get_integer( char** buf, int* pos );
	std::string _announce;
	bool _single;
	std::string _directory;
	BT_FILES _files;
	int _piece_length;
	std::vector< std::string > _pieces;
	int _size;
};

#endif

