#ifndef BITTORRENT_DOWNLOAD
#define BITTORRENT_DOWNLOAD

#include <qobject.h>
#include <qcstring.h>
#include <qstring.h>
#include <qmap.h>
#include <qvaluelist.h>
#include <qptrlist.h>

class QueryHitEntry;
class QUrlOperator;
class QNetworkOperation;
class QListViewItem;
class QPixmap;
class QTimer;
class DownloadViewItem;
class BitTorrent;
class BitTorrentPeerConnection;

class BitTorrentPeer
{
public:
	BitTorrentPeer( const QString& id, const QString& host, int port ) {
		_id = id;
		_host = host;
		_port = port;
	}
	QString _host;
	QString _id;
	int _port;
};

class BitTorrentDownload : public QObject
{
	Q_OBJECT
public:
	BitTorrentDownload( QueryHitEntry* );
	~BitTorrentDownload();
	void start();
	bool error();  //! Returns true on error.
	
signals:
	void doneTorrents( BitTorrentDownload* ); //! emitted when all torrents have been downloaded
	
public slots:
	void slotData( const QByteArray&, QNetworkOperation* );
	void slotFinished( QNetworkOperation* );

private slots:
	void retry();
	void slotPeerFinished();
	void slotPeerError( int );
	
protected:
	void setBall( QListViewItem*, QPixmap* );
	void createID();
	void finishedLoadingBittorrent();
	void finishedConnectingTracker();
	void downloadPiece( int piece );
	void setError( const QString& msg = "Error" );
	
	enum {
		NONE,
		LOADING_BITTORRENT,
		CONNECTING_TRACKER,
		NO_PEER,
		CONNECTING_PEER,
		ERROR
	};
	QueryHitEntry* _q;
	QUrlOperator* _url;
	QByteArray _data; //! content of the loaded torrent file or empty on error
	DownloadViewItem* _item;
	QString _id;
	int _state;
	int _start_piece_offset;
	BitTorrent* _bt;
	int _interval;
	int _downloaded;
	int _uploaded;
	int _port;
	QValueList< int > _pieces_left;
	QPtrList< BitTorrentPeer > _peers;
	QString _piece_url;
	int _retry_timer_countdown;
	QTimer* _retry_timer;
	int _current_peer;
	BitTorrentPeerConnection* _peer_connection;
	int _current_piece;
};

#endif

