#ifndef QUERYHITENTRY_H
#define QUERYHITENTRY_H

class QListViewItem;
#include <qstring.h>
#include <qstringlist.h>
#include <string>
#include "Address.h"

#if !defined(_WIN32)
using namespace std;
#endif

class QueryHitEntry
{
public:
	static const int  Busy;          //! status for GNUTELLA
	static const int  Ready;         //! status for GNUTELLA
	static const int  Firewalled;    //! status for GNUTELLA
	static const int  Unknown;       //! status for GNUTELLA

	static const int GNUTELLA;        //! File was found in the Gnet.
	static const int BITTORRENT;      //! File was found with BitTorrent search. (.torrent)
	static const int BITTORRENT_FILE; //! One file from BitTorrent file.
	
	QueryHitEntry();
	//! Constructor for Gnutella files.
	QueryHitEntry( const QString& filename, const Q_UINT32 size, const Q_UINT32 index, const Q_UINT32 speed, int status, Address& addr, std::string& id, QListViewItem *item, std::string& sha1 );
	//! Constructor for BitTorrent files.
	QueryHitEntry( const QString& filename, const Q_UINT32 size, const QString& torrentsource, int networktype = BITTORRENT );

	~QueryHitEntry();

	bool equal( QueryHitEntry* ) const;
	bool firewalled();
	int getNetworkType();
	void setDestinationFilename( const QString& );
	void setDestinationDirectory( const QString& );
	const QString& getDestinationFilename() const;
	const QString& getDestinationDirectory() const;
	//! Sets the item in the search list for this file. If "destr_delete" is true, the item is delete on deletion.
	void setSearchItem( QListViewItem*, bool destr_delete = true );
	const QString& getTorrentSource() const;
	void setTorrentSource( const QString& srcurl );
	
	std::string filename;    //! Filename of QueryHit
	Q_UINT32 size;           //! Size of the file in bytes.
	Q_UINT32 index;          //! Index of the file for. (GNUTELLA)
	Q_UINT32 speed;          //! Upload speed of remote host. (GNUTELLA)
	Address addr;            //! Addres of remote host. (GNUTELLA)
	int status;              //! Status of host. (GNUTELLA)
	std::string id;          //! Gnutella ID of remote host. (GNUTELLA)
	std::string sha1;        //! Hash value of file. (GNUTELLA)
	QListViewItem* item;     //! Pointer to item in QListView of search results.
	
protected:
	int networktype;
	QString _destination_file;
	QString _destination_dir;
	bool _search_item_delete;
	QString _torrent_source;  //! Link to the source torrent files. (BITTORRENT_FILE)
};

#endif
