#include "BitTorrent.h"
#include <cstdio>

#undef DEBUG
#ifdef DEBUG
#include <iostream>
#endif

BitTorrent::BitTorrent()
{
	_size = 0;
}

BitTorrent::~BitTorrent()
{
}

std::string* BitTorrent::get_string( char** buf, int* pos, int l )
{
	#ifdef DEBUG
		std::cout << "get string pos = " << *pos << std::endl;
	#endif
	int len = 0;
	while( *(*buf) != ':' ) {
		#ifdef DEBUG
			std::cout << *(*buf) << std::flush;
		#endif
		len = len * 10 + *(*buf) - '0';
		(*buf)++; (*pos)++;
	}
	(*buf)++; (*pos)++;
	#ifdef DEBUG
		std::cout << std::endl << "get string len = " << len << std::endl;
	#endif
	if( *pos + len > l ) {
		#ifdef DEBUG
			std::cout << "get string error" << std::endl;
		#endif
		return NULL;
	}
	std::string* ret = new std::string( *buf, len );
	#ifdef DEBUG
		if( len < 100 ) {
			std::cout << "got string [" << *ret << "]" << std::endl;
		} else {
			std::cout << "got string with length " << ret->size() << std::endl;
		}
	#endif
	*buf += len;
	*pos += len;
	return ret;
}

dictionary_t* BitTorrent::get_dictionary( char** buf, int* pos, int len )
{
	dictionary_t* d = new dictionary_t();
	while( *(*buf) != 'e' ) {
		if( *(*buf) >= '0' && *(*buf) <= '9' ) {
			std::string* s = get_string( buf, pos, len );
			#ifdef DEBUG
				std::cout << "got dictionary key: " << *s << std::endl;
			#endif
			elm* e = parse( buf, pos, len );
			if( ! e ) {
				delete s;
				return NULL;
			}
			d->insert( std::make_pair( s->c_str(), e ) );
			delete s;
		} else {
			#ifdef DEBUG
				std::cout << "not a string in dictionary as key" << std::endl;
			#endif
			delete d;
			return NULL;
		}
	}
	(*buf)++;
	(*pos)++;
	return d;
}

list_t* BitTorrent::get_list( char** buf, int* pos, int len )
{
	list_t* l = new list_t();
	while( *(*buf) != 'e' ) {
		elm* e = parse( buf, pos, len );
		if( ! e ) {
			delete l;
			return NULL;
		}
		l->push_back( e );
	}
	(*buf)++;
	(*pos)++;
	return l;
}

int* BitTorrent::get_integer( char** buf, int* pos )
{
	int* ret = new int();
	*ret = 0;
	while( *(*buf) != 'e' ) {
		*ret = *ret * 10 + *(*buf) - '0';
		(*buf)++; (*pos)++;
	}
	(*buf)++;
	(*pos)++;
	return ret;	
}

elm* BitTorrent::parse( const char* buf, int len )
{
	char* bu = (char*) malloc( len );
	char* b = bu;
	memcpy( bu, buf, len );
	int pos = 0;
	elm* e = parse( &bu, &pos, len );
	free( b );
	return e;
}

elm* BitTorrent::parse( char** buf, int* pos, int len )
{
	#ifdef DEBUG
		std::cout << "parse pos = " << *pos << std::endl;
	#endif
	elm* e = new elm();
	e->type = -1;
	if( *(*buf) == 'd' ) {
		#ifdef DEBUG
			std::cout << "  ->dictionary " << *(*buf) << std::endl;
		#endif
		(*buf)++; (*pos)++;
		e->val = (void*) get_dictionary( buf, pos, len );
		if( ! e->val ) {
			delete e;
			return NULL;
		}
		e->type = BT_DICTIONARY;
		return e;
	} else if( *(*buf) == 'l' ) {
		#ifdef DEBUG
			std::cout << "  ->list " << *(*buf) << std::endl;
		#endif
		(*buf)++; (*pos)++;
		e->val = (void*) get_list( buf, pos, len );
		if( ! e->val ) {
			delete e;
			return NULL;
		}
		e->type = BT_LIST;
		return e;
	} else if( *(*buf) == 'i' ) {
		#ifdef DEBUG
			std::cout << "  ->integer" << std::endl;
		#endif
		(*buf)++; (*pos)++;
		e->val = (void*) get_integer( buf, pos );
		if( ! e->val ) {
			delete e;
			return NULL;
		}
		#ifdef DEBUG
			std::cout << "  ->got value: " << *( (int*) e->val ) << std::endl;
		#endif
		e->type = BT_INTEGER;
		return e;
	} else if( *(*buf) >= '0' && *(*buf) <= '9' ) {
		#ifdef DEBUG
			std::cout << "  ->string" << *(*buf) << std::endl;
		#endif
		e->val = (void*) get_string( buf, pos, len );
		if( ! e->val ) {
			delete e;
			return NULL;
		}
		e->type = BT_STRING;
		return e;
	}
	delete e;
	#ifdef DEBUG
		std::cout << "  null" << std::endl;
	#endif
	return NULL;
}

elm* BitTorrent::get_dict_value( elm* e, const std::string& key )
{
	#ifdef DEBUG
		std::cout << "get_dict_value of " << key << std::endl;
	#endif
	elm* ret = NULL;
	dictionary_t* d = (dictionary_t*) e->val;
	if( d->find( key ) != d->end() ) {
		ret = (elm*) (*d)[ key ];
	} else {
		#ifdef DEBUG
			std::cout << "  ->key was not found" << std::endl;
			for( dictionary_t::iterator it = d->begin(); it != d->end(); ++it ) {
				std::cout << "  ->key in: " << it->first << std::endl;
			}
		#endif
	}
	return ret;
}

void BitTorrent::setAnnounce( const std::string& a )
{
	_announce = a;
}

const std::string& BitTorrent::getAnnounce() const
{
	return _announce;
}

void BitTorrent::setSingleFile( bool s )
{
	_single = s;
}

bool BitTorrent::singleFile() const
{
	return _single;
}

void BitTorrent::addFile( const std::string& fname, int len )
{
	_files.push_back( std::make_pair( fname, len ) );
}

void BitTorrent::setDirectory( const std::string& dir )
{
	_directory = dir;
}

const std::string& BitTorrent::getDirectory() const
{
	return _directory;
}

void BitTorrent::setPieceLength( int l )
{
	_piece_length = l;
}

int BitTorrent::getPieceLength() const
{
	return _piece_length;
}

void BitTorrent::setPieces( const std::string& s )
{
	for( int i = 0; i < s.size(); i += 20 ) {
		_pieces.push_back( s.substr( i, 20 ) );
	}
}

const std::string& BitTorrent::getPiece( int n ) const
{
	return _pieces[ n ];
}

int BitTorrent::pieces() const
{
	return _pieces.size();
}

int BitTorrent::size() const
{
	return _size;
}

void BitTorrent::addSize( int len )
{
	_size += len;
}

int BitTorrent::getLength( const std::string& file ) const
{
	int ret = -1;
	for( BT_FILES::const_iterator it = _files.begin(); it != _files.end(); ++it ) {
		if( it->first == file ) {
			ret = it->second;
			break;
		}
	}
	return ret;
}

bool BitTorrent::getPieces( const std::string& file, int& startpiece, int& endpiece, int& startpieceoffset ) const
{
	bool ret = false;
	unsigned bytes = 0;
	for( BT_FILES::const_iterator it = _files.begin(); it != _files.end(); ++it ) {
		if( it->first == file ) {
			startpiece = bytes / getPieceLength();
			startpieceoffset = bytes - startpiece * getPieceLength();
			int required;
			if( it->second % getPieceLength() == 0 ) {
				required = it->second / getPieceLength();
			} else {
				required = it->second / getPieceLength() + 1;
			}
			if( required > 0 ) {
				endpiece = startpiece + required - 1;
			} else {
				endpiece = startpiece;
			}
			ret = true;
			break;
		} else {
			bytes += it->second;
		}
	}
	return ret;
}

BitTorrent* BitTorrent::decode( const char* buf, int len )
{
	#ifdef DEBUG
		std::cout << "bt_decode" << std::endl;
	#endif
	char* bu = (char*) malloc( len );
	char* b = bu;
	memcpy( bu, buf, len );
	int pos = 0;
	#ifdef DEBUG
		std::cout << "bt_decode length = " << len << std::endl;
	#endif
	elm* e = parse( &bu, &pos, len );
	free( b );
	BitTorrent* bt = new BitTorrent();
	if( e && e->type == BT_DICTIONARY ) {
		elm* v = get_dict_value( e, "announce" );
		if( v && v->type == BT_STRING && v->val ) {
			bt->setAnnounce( *( (std::string*) v->val ) );
		} else {
			delete bt;
			delete e;
			return NULL;
		}
		v = get_dict_value( e, "info" );
		if( v && v->type == BT_DICTIONARY ) {
			elm* n;
			n = get_dict_value( v, "piece length" );
			if( n && n->type == BT_INTEGER && n->val ) {
				bt->setPieceLength( *( (int*) n->val ) );
			} else {
				#ifdef DEBUG
					std::cout << "bt_decode no piece length was found" << std::endl;
				#endif
				delete bt;
				delete e;
				return NULL;
			}
			n = get_dict_value( v, "pieces" );
			if( n && n->type == BT_STRING && n->val ) {
				bt->setPieces( *( (std::string*) n->val ) );
			} else {
				#ifdef DEBUG
					std::cout << "bt_decode no pieces was found" << std::endl;
				#endif
				delete bt;
				delete e;
				return NULL;
			}
			if( ( n = get_dict_value( v, "length" ) ) ) {  // single file
				bt->setSingleFile( true );
				int len = -1;
				std::string fname;
				elm* h = get_dict_value( v, "length" );
				if( h && h->type == BT_INTEGER ) {
					len = *( (int*) h->val );
					bt->addSize( len );
					h = get_dict_value( v, "name" );
					if( h && h->type == BT_STRING ) {
						fname = std::string( ( (std::string*) h->val )->c_str() );
						bt->addFile( fname, len );
					} else {
						delete bt;
						bt = NULL;
					}
				} else {
					delete bt;
					bt = NULL;
				}
			} else {
				bt->setSingleFile( false );
				elm* h = get_dict_value( v, "name" );
				if( h && h->type == BT_STRING ) {
					bt->setDirectory( ( (std::string*) h->val )->c_str() );
					v = get_dict_value( v, "files" );
					if( v && v->type == BT_LIST ) {
						list_t* l = (list_t*) v->val;
						for( list_t::iterator it = l->begin(); it != l->end(); ++it ) {
							elm* h = *it;
							elm* p = get_dict_value( h, "length" );
							int len = -1;
							if( p && p->type == BT_INTEGER ) {
								len = *( (int*) p->val );
								bt->addSize( len );
								p = get_dict_value( h, "path" );
								if( p && p->type == BT_LIST ) {
									std::string fname;
									list_t* g = (list_t*) p->val;
									for( list_t::iterator i = g->begin(); i != g->end(); ++i ) {
										elm* k = *i;
										if( k && k->type == BT_STRING ) {
											fname = std::string( ( (std::string*) k->val )->c_str() );
										}
									}
									if( ! fname.empty() ) {
										bt->addFile( fname, len );
									} else {
										delete bt;
										bt = NULL;
										break;
									}
								} else {
									delete bt;
									bt = NULL;
									break;
								}
							} else {
								delete bt;
								bt = NULL;
								break;
							}
						}
					} else {
						delete bt;
						bt = NULL;
					}
				} else {
					delete bt;
					bt = NULL;
				}
			}
		} else { // info no dictionary or key not found
			#ifdef DEBUG
				std::cout << "parse error (1)" << std::endl;
			#endif
			delete bt;
			bt = NULL;
		}
	} else { // no dictionary
		#ifdef DEBUG
			std::cout << "parse error" << std::endl;
		#endif
		delete bt;
		bt = NULL;
	}
	if( e ) {
		delete e;
	}
	return bt;
}

BitTorrent* BitTorrent::bt_decode_file( const char* filename ) 
{
	char* buf = (char*) malloc( MAXSIZE_BT_FILE );
	FILE* f = fopen( filename, "rb");
	int read = fread( buf, 1, MAXSIZE_BT_FILE, f );
	fclose( f );
	BitTorrent* bt = NULL;
	if( ! read ) {
		return bt;
	}
	bt = decode( buf, read );
	free( buf );
	return bt;
}
