#include "BitTorrentDownload.h"
#include "QueryHitEntry.h"
#include "QtellaSub.h"
#include "DownloadViewItem.h"
#include "WidgetDownloads.h"
#include "Icon.h"
#include "StringManipulation.h"
#include "BitTorrent.h"
#include "BitTorrentPeerConnection.h"
#include <qlistview.h>
#include <qurloperator.h>
#include <qnetworkprotocol.h>
#include <qtimer.h>
#include <qcstring.h>
#include <string>
#include <stdio.h>
#include <cstdlib>

#define RETRY_TIMEOUT 30
#define RETRY_TIMEOUT_GET_PEER 180

BitTorrentDownload::BitTorrentDownload( QueryHitEntry* q )
{
#warning DEBUG
	q->setTorrentSource( "http://h6513.serverkompetenz.net/torrent/mytest.torrent" );
	q->filename = "ICOM-Group1.gif";

	//q->setTorrentSource( "http://66.90.75.92/suprnova//torrents/1322/46744c673d2c1597f9eae7f3e5d27db7cc2fa516.torrent" );
	//q->filename = "aleksei glyzin the best.by sagitoff & sharemusic.ru.rar";
	
	_q = q;
	_url = NULL;
	_item = new DownloadViewItem( QtellaSub::getInstance()->_widget_downloads->ui_listview_download );
	_item->setColumn( DownloadViewItem::FILENAME, q->filename.c_str() );
	_item->setColumn( DownloadViewItem::STATUS, "None" );
	_item->setColumn( DownloadViewItem::HOST, "BitTorrent" );
	_item->setColumn( DownloadViewItem::CLIENT, "BitTorrent" );
	_state = NONE;
	QString s;
	s.setNum( q->size );
	s = StringManipulation::insertDelimiter( s, ',', 3 );
	_item->setColumn( DownloadViewItem::SIZE, s );
	setBall( q->item, Icon::getInstance()->getYellowBall() );
	createID();
	_bt = NULL;
	_downloaded = _uploaded = 0;
	_port = 6881;
	_retry_timer = NULL;
}

void BitTorrentDownload::createID()
{
	QString chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
	_id = "QTEL00";
	for( int i = 0; i < 14; ++i ) {
		_id += chars[ rand() % chars.length() ];
	}
}

void BitTorrentDownload::setBall( QListViewItem* item, QPixmap* p )
{
	item->setPixmap( 2, *p );
	QListViewItem* par = item->parent();
}

void BitTorrentDownload::setError( const QString& msg )
{
	_item->setColumn( DownloadViewItem::STATUS, msg );
	_state = ERROR;
	if( _q->item ) {
		setBall( _q->item, Icon::getInstance()->getRedBall() );
	}
}

BitTorrentDownload::~BitTorrentDownload()
{
	if( _url ) {
		_url->stop();
		delete _url;
	}
	if( _item ) {
		delete _item;
	}
	if( _retry_timer ) {
		disconnect( _retry_timer );
		delete _retry_timer;
	}
}

// is called by the BitTorrentDownloadManager
void BitTorrentDownload::start()
{
	qDebug( "BitTorrentDownload::start() %s", _q->getTorrentSource().ascii() );
	_item->setColumn( DownloadViewItem::STATUS, "Connecting" );
	_state = LOADING_BITTORRENT;
	_url = new QUrlOperator( _q->getTorrentSource() );
	connect( _url, SIGNAL( data( const QByteArray&, QNetworkOperation* ) ), SLOT( slotData( const QByteArray&, QNetworkOperation* ) ) );
	connect( _url, SIGNAL( finished( QNetworkOperation* ) ), SLOT( slotFinished( QNetworkOperation* ) ) );
	_url->get();
}

void BitTorrentDownload::slotData( const QByteArray& b, QNetworkOperation* o )
{
	qDebug( "BitTorrentDownload::data()" );
	if( _state == LOADING_BITTORRENT || _state == CONNECTING_TRACKER ) {
		int off = _data.size();
		_item->setColumn( DownloadViewItem::STATUS, "Retrieving" );
		_data.resize( _data.size() + b.size() );
		for( int i = 0; i < b.size(); ++i ) {
			_data[ i + off ] = b[ i ];
		}
	}
}

void BitTorrentDownload::retry()
{
	if( _url ) {
		_url->stop();
		delete _url;
		_url = NULL;
	}
	if( _retry_timer_countdown > 0 ) {
		if( ! _retry_timer ) {
			_retry_timer = new QTimer();
			connect( _retry_timer, SIGNAL( timeout() ), SLOT( retry() ) );
			_retry_timer->start( 1000 );
		} else {
			_retry_timer_countdown--;
		}
		QString c = _item->text( DownloadViewItem::STATUS );
		if( c.find( "-" ) != -1 ) {
			c = c.left( c.find( "-" ) );
		}
		QString msg = c + "- " + QString::number( _retry_timer_countdown ) + "s";
		_item->setColumn( DownloadViewItem::STATUS, msg );
		return;
	} else {
		if( _retry_timer ) {
			disconnect( _retry_timer );
			delete _retry_timer;
			_retry_timer = NULL;
		}
	}
	qDebug( "BitTorrentDownload::retry()" );
	if( _state == LOADING_BITTORRENT ) {
		_item->setColumn( DownloadViewItem::STATUS, "Connecting" );
		_url = new QUrlOperator( _q->getTorrentSource() );
	} else if( _state == CONNECTING_TRACKER ) {
		_item->setColumn( DownloadViewItem::STATUS, "Connecting" );
		_url = new QUrlOperator( _piece_url );
	} else if( _state == NO_PEER ) {
		_item->setColumn( DownloadViewItem::STATUS, "Connecting" );
		_url = new QUrlOperator( _piece_url );
		_state = CONNECTING_TRACKER;
	}
	setBall( _q->item, Icon::getInstance()->getYellowBall() );
	_data.resize( 0 );
	connect( _url, SIGNAL( data( const QByteArray&, QNetworkOperation* ) ), SLOT( slotData( const QByteArray&, QNetworkOperation* ) ) );
	connect( _url, SIGNAL( finished( QNetworkOperation* ) ), SLOT( slotFinished( QNetworkOperation* ) ) );
	_url->get();
}

void BitTorrentDownload::slotFinished( QNetworkOperation* o )
{
	qDebug( "BitTorrentDownload::finished %d", o->errorCode() );
	if( o->errorCode() != QNetworkProtocol::NoError ) {
		if( _state == LOADING_BITTORRENT || _state == CONNECTING_TRACKER ) {
			_retry_timer_countdown = RETRY_TIMEOUT;
			_item->setColumn( DownloadViewItem::STATUS, "Retry -" );
			retry();
		} else {
			setError();
		}
	} else {
		if( _state == LOADING_BITTORRENT ) {
			qDebug( "BitTorrentDownload::finished() LOADING_BITTORRENT" );
			finishedLoadingBittorrent();
		} else if( _state == CONNECTING_TRACKER ) {
			qDebug( "BitTorrentDownload::finished() CONNECTING_TRACKER" );
			finishedConnectingTracker();
		}
	}
}

void BitTorrentDownload::finishedConnectingTracker()
{
	qDebug( "BitTorrentDownload::finishedConnectingTracker() rsp: <%s>", (const char*) _data );
	elm* e = BitTorrent::parse( _data, _data.size() );
	if( e && e->type == BT_DICTIONARY ) {
		elm* v = BitTorrent::get_dict_value( e, "failure reason" );
		if( v && v->val && v->type == BT_STRING) {
			std::string* s = (std::string*) v->val;
			qDebug( "BitTorrentDownload::finishedConnectingTracker() failure: <%s>", (*s).c_str() );
			setError();
			emit doneTorrents( this );
			return;
		} else {
			v = BitTorrent::get_dict_value( e, "interval" );
			if( v && v->val && v->type == BT_INTEGER ) {
				_interval = *( (int*) v->val );
				qDebug( "BitTorrentDownload::finishedConnectingTracker() interval %d", _interval );
				v = BitTorrent::get_dict_value( e, "peers" );
				if( v && v->val && v->type == BT_LIST ) {
					std::list< elm* >* l = (std::list< elm* >*) v->val;
					bool error = false;
					for( std::list< elm* >::iterator it = l->begin(); it != l->end(); ++it ) {
						if( (*it) && (*it)->val && (*it)->type == BT_DICTIONARY ) {
							v = BitTorrent::get_dict_value( *it, "peer id" );
							QString peerid;
							QString host;
							int port;
							if( v && v->val && v->type == BT_STRING ) {
								peerid = ( (std::string*) v->val )->c_str();
							} else {
								error = true;
								break;
							}
							v = BitTorrent::get_dict_value( *it, "ip" );
							if( v && v->val && v->type == BT_STRING ) {
								host = ( (std::string*) v->val )->c_str();
							} else {
								error = true;
								break;
							}
							v = BitTorrent::get_dict_value( *it, "port" );
							if( v && v->val && v->type == BT_INTEGER ) {
								port = *( (int*) v->val );
							} else {
								error = true;
								break;
							}
							qDebug( "port <%d> ip <%s> peer id <%s>", port, host.ascii(), peerid.ascii() );
							if( peerid == _id ) {
								qDebug( "ignoring my id" );
							} else {
								_peers.append( new BitTorrentPeer( peerid, host, port ) );
							}
						} else {
							error = true;
							break;
						}
					}
					if( ! error ) {
						qDebug( "BitTorrentDownload::finishedConnectingTracker() peers found: %d", _peers.count() );
						delete e;
						if( _peers.count() > 0 ) {
							if( _url ) {
								_url->stop();
								delete _url;
								_url = NULL;
							}
							_current_peer = 0;
							_item->setColumn( DownloadViewItem::STATUS, "Connecting" );
							_state = CONNECTING_PEER;
							_peer_connection = new BitTorrentPeerConnection( _peers.at( 0 )->_host, _peers.at( 0 )->_port, _peers.at( 0 )->_id, _id );
							connect( _peer_connection, SIGNAL( error( int ) ), SLOT( slotPeerError( int ) ) );
							connect( _peer_connection, SIGNAL( finished() ), SLOT( slotPeerFinished() ) );
							std::string s = _bt->getPiece( _current_piece );
							QByteArray b;
							b.assign( s.data(), s.size() );
							_peer_connection->start( b, _bt->getPieceLength() );
						} else {
							_item->setColumn( DownloadViewItem::STATUS, "No peer -" );
							_state = NO_PEER;
							_retry_timer_countdown = RETRY_TIMEOUT_GET_PEER;
							retry();
						}
						return;
					}
				}
			}
		}
	}
	if( e ) {
		delete e;
	}
	qDebug( "BitTorrentDownload::finishedConnectingTracker() parse error" );
	setError();
	emit doneTorrents( this );
}

void BitTorrentDownload::finishedLoadingBittorrent()
{
	_item->setColumn( DownloadViewItem::STATUS, "Loaded" );
	_bt = BitTorrent::decode( _data, _data.size() );
	_data.resize( 0 );
	if( _bt ) {
		qDebug( "BitTorrentDownload::finished announce <%s>", _bt->getAnnounce().c_str() );
		qDebug( "BitTorrentDownload::finished piecelength <%d>", _bt->getPieceLength() );
		_url->stop();
		delete _url;
		_url = NULL;
		// what pieces do we need to get the requested file?
		int start_piece;
		int end_piece;
		if( _bt->getPieces( _q->filename.c_str(), start_piece, end_piece, _start_piece_offset ) ) {
			int len = _bt->getLength( _q->filename.c_str() );
			qDebug( "BitTorrentDownload::finished startpiece: %d", start_piece );
			qDebug( "BitTorrentDownload::finished endpiece: %d", end_piece );
			qDebug( "BitTorrentDownload::finished startpieceoffset: %d", _start_piece_offset );
			for( int i = start_piece; i <= end_piece; ++i ) {
				_pieces_left.append( i );
			}
			downloadPiece( start_piece );
		} else {
			setError( "Invalid" );
			emit doneTorrents( this );
		}
	} else {
		setError( "Invalid" );
		emit doneTorrents( this );
	}
}

void BitTorrentDownload::downloadPiece( int piece )
{
	_current_piece = piece;
	qDebug( "BitTorrentDownload::downloadPiece %d (%d)", piece, _bt->pieces() );
	_item->setColumn( DownloadViewItem::STATUS, "Connecting" );
	_state = CONNECTING_TRACKER;
	_piece_url = _bt->getAnnounce().c_str();
	QString p;
	std::string s = _bt->getPiece( piece );
	for( int i = 0; i < s.size(); ++i ) {
		unsigned char c = s[ i ];
		QString tmp;
		tmp = tmp.sprintf( "%02X", (int) c );
		p += "%" + tmp.upper();
	}
	QString left = QString::number( _pieces_left.size() * _bt->getPieceLength() - _downloaded );
	_piece_url += "?info_hash=" + p + "&peer_id=" + _id + "&port=" + QString::number( _port ) + "&uploaded=" + QString::number( _uploaded ) + 
		"&downloaded=" + QString::number( _downloaded ) + "&left=" + left;
	qDebug( "BitTorrentDownload::downloadPiece url: <%s>", _piece_url.ascii() );
	_url = new QUrlOperator( _piece_url );
	connect( _url, SIGNAL( data( const QByteArray&, QNetworkOperation* ) ), SLOT( slotData( const QByteArray&, QNetworkOperation* ) ) );
	connect( _url, SIGNAL( finished( QNetworkOperation* ) ), SLOT( slotFinished( QNetworkOperation* ) ) );
	_url->get();
}

bool BitTorrentDownload::error()
{
	return _state == ERROR;
}

void BitTorrentDownload::slotPeerFinished() // state = CONNECTING_PEER
{
	qDebug( "BitTorrentDownload::slotPeerFinished()" );
}

void BitTorrentDownload::slotPeerError( int err ) // state = CONNECTING_PEER
{
	qDebug( "BitTorrentDownload::slotPeerError() <%d>", err );
#warning TODO try next peer
}

