#include "BitTorrentSearch.h"
#include <qurloperator.h>
#include <qcstring.h>
#include <qregexp.h>

BitTorrentSearch::BitTorrentSearch( const QString& query, const QString& searchid )
{
	_searchid = searchid;
	_query = query;
}

BitTorrentSearch::~BitTorrentSearch()
{
	if( _op ) {
		_op->stop();
		delete _op;
	}
}

void BitTorrentSearch::start()
{
	_op = new QUrlOperator( QUrlOperator( "http://www.btbot.com/" ), "/search.bt?q=" + _query );
	connect( _op, SIGNAL( data( const QByteArray&, QNetworkOperation* ) ), SLOT( btData( const QByteArray&, QNetworkOperation* ) ) );
	connect( _op, SIGNAL( finished( QNetworkOperation* ) ), SLOT( btFinished( QNetworkOperation* ) ) );
	_op->get();
}

QValueList< TORRENTS > BitTorrentSearch::getTorrents()
{
	return _torrents;
}

void BitTorrentSearch::btFinished( QNetworkOperation* n )
{
	QRegExp r( "<table\\sclass=txt><tr><td\\svalign=top><b>(.*)<br></td></tr></table>" );
	r.setMinimal( true );
	int pos = 0;
	while( ( pos = r.search( _data, pos ) ) != -1 ) {
		QString s( _data.mid( pos, r.matchedLength() ) );
		TORRENTS t = decode( s );
		if( t.first.size() > 0 ) {
			_torrents.push_back( t );
		}
		pos += r.matchedLength();
	}
	r.setPattern( "<a.href=(search.bt.q=.*)\\sstyle.*>(.*)</a>" );
	r.setMinimal( true );
	pos = 0;
	while( ( pos = r.search( _data, pos ) ) != -1 ) {
		QStringList l = r.capturedTexts();
		if( l.size() > 2 && l[ 2 ] == "next" ) {
			_query = l[ 1 ];
			_data = "";
			_op->stop();
			delete _op;
			start();
			return;
		}
		pos += r.matchedLength();
	}
	emit finished( _searchid );
}

void BitTorrentSearch::btData( const QByteArray& b, QNetworkOperation* n )
{
	_data.append( b );
}

TORRENTS BitTorrentSearch::decode( const QString& s ) // decodes one hit
{
	TORRENTS ret;
	QString sub;
	if( s.find( "<font" ) != -1 ) {
		QStringList torrent_files;                      // all torrent files
		QValueList< QPair< QString, QString > > c;      // files and filesizes
		sub = s.left( s.find( "<font" ) );
		QRegExp r( "<a\\shref=(.*)\\sclass=ll" );
		r.setMinimal( true );
		int pos = 0;
		while( ( pos = r.search( sub, pos ) ) != -1 ) {
			QStringList capt = r.capturedTexts();
			if( capt.size() > 1 ) {
				QString x( capt[ 1 ] );
				torrent_files.append( x );
				QRegExp rf( "<table.*<td.*<td>&nbsp;&nbsp;&nbsp;</td><td>(.*)</td>" );  // grab the filenames
				rf.setMinimal( true );
				QString right( s.mid( s.find( "<font" ) ) );
				if( rf.search( right ) != -1 ) {
					capt = rf.capturedTexts();
					if( capt.size() > 1 ) {
						x = capt[ 1 ];
						x.replace( QRegExp( "<b>" ), "" );
						x.replace( QRegExp( "</b>" ), "" );
						QStringList files = QStringList::split( "<br>", x );
						QRegExp rf( "<table.*<td.*>(.*)<td>&nbsp;&nbsp;&nbsp;</td><td>" );  // grab the filesizes
						rf.setMinimal( true );
						if( rf.search( right ) != -1 ) {
							capt = rf.capturedTexts();
							if( capt.size() > 1 ) {
								x = capt[ 1 ];
								x.replace( QRegExp( "<b>" ), "" );
								x.replace( QRegExp( "</b>" ), "" );
								QStringList sizes = QStringList::split( "<br>", x );
								QStringList::Iterator fit = files.begin();
								QStringList::Iterator sit = sizes.begin();
								while( fit != files.end() && sit != files.end() ) {
									c.push_back( qMakePair( *fit, *sit ) );
									++fit; ++sit;
								}
							}
						}
					}
				}
			}
			pos += r.matchedLength();
		}
		ret = qMakePair( torrent_files, c );
	}
	return ret;
}
