#include "../include/Messenger.h"
#include "MessengerThread.h"
#include <yahoo2.h>
#include <qlineedit.h>
#include <qcolor.h>
#include <qtextedit.h>

/* 
 *  Constructs a Messenger which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
Messenger::Messenger( QWidget* parent,  const char* name, WFlags fl )
    : BMessenger( parent, name, fl )
{
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Messenger::~Messenger()
{
    // no need to delete child widgets, Qt does it all for us
}

void Messenger::setMyId( int id )
{
	_id = id;
}

void Messenger::setWho( const QString& who )
{
	_who = who;
	setCaption( who );
}

void Messenger::setMe( const QString& me )
{
	_me = me;
}

void Messenger::slotPressedEnter()
{
	QString msg( lineedit_send->text() );
	lineedit_send->setText( "" );
	textedit->setColor( QColor( 0x20, 0x20, 0xa0 ) );
	textedit->insert( _me + ": " );
	textedit->setColor( QColor( 0, 0, 0 ) );
	textedit->insert( msg + "\n" );
	yahoo_send_im( _id, _me.ascii(), _who.ascii(), msg.ascii(), 0 );
}

void Messenger::addMessage( const QString& from, const QString& msg )
{
	textedit->setColor( QColor( 0xa0, 0x20, 0x20 ) );
	textedit->insert( from + ": " );
	textedit->setColor( QColor( 0, 0, 0 ) );
	textedit->insert( msg + "\n" );
}

const QString& Messenger::getWho() const
{
	return _who;
}

