#include "../include/MessengerContacts.h"
#include "MessengerThread.h"
#include "MessengerLogin.h"
#include "MessengerStatus.h"
#include "Messenger.h"
#include "Icon.h"
#include "QtellaSub.h"
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qregexp.h>
#include <qwidget.h>
#include <qworkspace.h>
#include <yahoo2_types.h>
#include <yahoo2.h>

MessengerContacts* MessengerContacts::_instance = NULL;
bool MessengerContacts::_invisible = false;
int MessengerContacts::_id = -1;

/* 
 *  Constructs a MessengerContacts which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
MessengerContacts::MessengerContacts( QWidget* parent,  const char* name, WFlags fl )
    : BMessengerContacts( parent, name, fl )
{
	connect( (QObject*) pushbutton_connect, SIGNAL( clicked() ), SLOT( connectYh() ) );
	connect( (QObject*) pushbutton_disconnect, SIGNAL( clicked() ), SLOT( disconnectYh() ) );
	_instance = this;
	_mstatus = NULL;
}

MessengerContacts* MessengerContacts::instance()
{
	return _instance;
}

void MessengerContacts::init( const QString& userid, const QString& passwd, bool invisible )
{
	_user = userid;
	_passwd = passwd;
	_invisible = invisible;
	_ml = NULL;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
MessengerContacts::~MessengerContacts()
{
    // no need to delete child widgets, Qt does it all for us
}

void MessengerContacts::connectYh()
{
	if( ! _ml ) {
		_ml = new MessengerLogin();
		connect( (QObject*) _ml->pushbutton_connect, SIGNAL( clicked() ), (QObject*) _ml, SLOT( accept() ) );
	}
	_ml->lineedit_id->setText( _user );
	_ml->lineedit_passwd->setText( _passwd );
	_ml->checkbox_invisible->setChecked( _invisible );
	if( _ml->exec() == QDialog::Accepted ) {
		_user = _ml->lineedit_id->text();
		_passwd = _ml->lineedit_passwd->text();
		_invisible = _ml->checkbox_invisible->isChecked();
		textlabel_status->setText( "Connecting..." );
		_messenger_thread = new MessengerThread( _user, _passwd, _invisible, this );
		_messenger_thread->start();
		pushbutton_connect->setEnabled( false );
		pushbutton_disconnect->setEnabled( true );
	}
}

void MessengerContacts::connected( int id )
{
	_id = id;
	if( ! _invisible ) {
		_instance->pushbutton_change->setEnabled( true );
	}
	_instance->textlabel_status->setText( ( _instance->_invisible ? "Invisible" : "Available" ) );
}

void MessengerContacts::connectError()
{
	_instance->pushbutton_change->setEnabled( false );
	_instance->pushbutton_connect->setEnabled( true );
	_instance->pushbutton_disconnect->setEnabled( false );
	_instance->textlabel_status->setText( "Error" );
	_instance->listview->clear();
}

void MessengerContacts::disconnectYh()
{
	_id = -1;
	_instance->listview->clear();
	_messenger_thread->disconnect();
	_instance->pushbutton_connect->setEnabled( true );
	_instance->pushbutton_disconnect->setEnabled( false );
}

void MessengerContacts::addContact( const QString& yid, const QString& group, const QString& status )
{
	qDebug( "MessengerContacts::addContact() got yid: %s", yid.ascii() );
	QListViewItem* i = new QListViewItem( listview, yid, status );
	if( status.lower() == "offline" ) {
		i->setPixmap( 0, *( Icon::getInstance()->getRedBall() ) );
	} else {
		i->setPixmap( 0, *( Icon::getInstance()->getGreenBall() ) );
	}
}

void MessengerContacts::slotItemSelected( QListViewItem* i )
{
	qDebug( "item selected %s", i->text( 0 ).ascii() );
	Messenger* m = new Messenger( (QWidget*) QtellaSub::getInstance()->getWorkspace(), i->text( 0 ).ascii() );
	m->setMyId( _id );
	m->setWho( i->text( 0 ) );
	m->setMe( _user );
	m->show();
	_messengers.append( m );
}

void MessengerContacts::receivedMessage( char* from, char* msg )
{
	QString from_( from );
	QString msg_( msg );
	QRegExp r( "<.*>" );
	r.setMinimal( true );
	msg_.replace( r, "" );
	for( int i = 0; i < _messengers.count(); ++i ) {
		if( ( _messengers.at( i ) )->getWho() == from_ ) {
			( _messengers.at( i ) )->addMessage( from_, msg_ );
			break;
		}
	}
}

void MessengerContacts::updateContact( const QString& yid, int status, const QString& msg )
{
	QString s;
	switch( status ) {
		case YAHOO_STATUS_AVAILABLE:
			s = "Available";
			break;
		case YAHOO_STATUS_BRB:
			s = "BRB";
			break;
		case YAHOO_STATUS_BUSY:
			s = "Busy";
			break;
		case YAHOO_STATUS_NOTATHOME:
			s = "Not at Home";
			break;
		case YAHOO_STATUS_NOTATDESK:
			s = "Not at Desk";
			break;
		case YAHOO_STATUS_NOTINOFFICE:
			s = "Not in Office";
			break;
		case YAHOO_STATUS_ONPHONE:
			s = "On the Phone";
			break;
		case YAHOO_STATUS_ONVACATION:
			s = "On Vacation";
			break;
		case YAHOO_STATUS_OUTTOLUNCH:
			s = "Stepped out";
			break;
		case YAHOO_STATUS_CUSTOM:
			s = msg;
			break;
		case YAHOO_STATUS_IDLE:
			s = "Idle";
			break;
		case YAHOO_STATUS_OFFLINE:
			s = "offline";
			break;
	}
	for( QListViewItem* i = listview->firstChild(); i; i = i->nextSibling() ) {
		if( i->text( 0 ) == yid ) {
			i->setText( 1, s );
			if( s == "offline" ) {
				i->setPixmap( 0, *( Icon::getInstance()->getRedBall() ) );
			} else {
				i->setPixmap( 0, *( Icon::getInstance()->getGreenBall() ) );
			}
		}
	}
}

void MessengerContacts::slotChangeStatus()
{
	if( ! _mstatus ) {
		_mstatus = new MessengerStatus();
		connect( (QObject*) _mstatus->pushbutton, SIGNAL( clicked() ), _mstatus, SLOT( accept() ) );
	}
	if( _mstatus->exec() == QDialog::Accepted ) {
		int away = 0;
		if( _mstatus->checkbox_busy->isChecked() ) {
			away = 1;
		}
		const char* msg = NULL;
		int status = 0;
		if( _mstatus->lineedit_status->text().length() > 0 ) {
			status = YAHOO_STATUS_CUSTOM;
			msg = _mstatus->lineedit_status->text().ascii();
			textlabel_status->setText( _mstatus->lineedit_status->text() );
		} else {
			status = _mstatus->combobox->currentItem();
			textlabel_status->setText( _mstatus->combobox->currentText() );
		}
		yahoo_set_away( _id, (yahoo_status) status, msg, away );
	}
}

