#include "MessengerThread.h"
#include "MessengerContacts.h"
#include <iostream>
#include <netdb.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <yahoo2.h>
#include <yahoo2_callbacks.h>
#include <yahoo_list.h>

MessengerContacts* MessengerThread::_messenger_contacts = NULL;

#define y_new0(type, n) (type *)calloc((n), sizeof(type))

struct connect_callback_data {
    yahoo_connect_callback callback;
    void * callback_data;
    int id;
    int tag;
};

YList *connections = NULL;

struct _conn {
    int tag;
    int fd;
    int id;
    yahoo_input_condition cond;
    void *data;
    int remove;
}; 

static int connection_tags=0;

bool _buddies_need_update = true;

void yahoo_callback(struct _conn *c, yahoo_input_condition cond)
{
    int ret=1;
    char buff[1024]={0};
    if(cond & YAHOO_INPUT_READ)
        ret = yahoo_read_ready(c->id, c->fd, c->data);
    if(ret>0 && cond & YAHOO_INPUT_WRITE)
        ret = yahoo_write_ready(c->id, c->fd, c->data);
}

MessengerThread::MessengerThread( const QString& user, const QString& passwd, bool invisible, MessengerContacts* mc )
{
	_user = user;
	_passwd = passwd;
	_messenger_contacts = mc;
	_buddies_need_update = true;
	_invisible = invisible;
}

MessengerThread::~MessengerThread()
{
}

void MessengerThread::disconnect()
{
	yahoo_close( _id );
	_quit = true;
}

void MessengerThread::run()
{
	qDebug( "Messenger::run() connecting..." );
	_quit = false;
	_id = yahoo_init( _user.ascii(), _passwd.ascii() );
	qDebug( "Messenger::run() id = %d", _id );
	yahoo_login( _id, ( _invisible ? YAHOO_STATUS_INVISIBLE : YAHOO_STATUS_AVAILABLE ) );
    int lfd=0;  
    fd_set inp, outp;
    struct timeval tv;
    YList *l = connections;
	while( ! _quit ) {
        FD_ZERO(&inp);
        FD_ZERO(&outp);
        tv.tv_sec=1;
        tv.tv_usec=0;
        lfd=0;
        for(l=connections; l; ) {
            struct _conn *c = (struct _conn*) l->data;
            if(c->remove) {
                YList *n = y_list_next(l);
                connections = y_list_remove_link(connections, l);
                y_list_free_1(l);
                free(c);
                l=n;
            } else {
                if(c->cond & YAHOO_INPUT_READ)
                    FD_SET(c->fd, &inp);
                if(c->cond & YAHOO_INPUT_WRITE)
                    FD_SET(c->fd, &outp);
                if(lfd < c->fd)
                    lfd = c->fd;
                l = y_list_next(l);
            }
        }
        select(lfd + 1, &inp, &outp, NULL, &tv);
        for(l = connections; l; l = y_list_next(l)) {
            struct _conn *c = (struct _conn*) l->data;
            if(c->remove)
                continue;
            if(FD_ISSET(c->fd, &inp))
                yahoo_callback(c, YAHOO_INPUT_READ);
            if(FD_ISSET(c->fd, &outp))
                yahoo_callback(c, YAHOO_INPUT_WRITE);
        }
    }
	qDebug( "Messenger::run() quit." );
}

//! Called when the login process is complete
extern void ext_yahoo_login_response( int id, int succ, char *url )
{
	qDebug( "Messenger::ext_yahoo_login_response %d %d %s", id, succ, url );
	if( succ == YAHOO_LOGIN_OK ) {
		MessengerContacts::connected( id );
	} else {
		MessengerContacts::connectError();
	}
}

//! Called when the contact list is got from the server
void ext_yahoo_got_buddies( int id, YList* buds ) {
	qDebug( "ext_yahoo_got_buddies" );
	for( ; buds; buds = buds->next ) {
		struct yahoo_buddy *bud = (struct yahoo_buddy*) buds->data;
		qDebug( "yid %s, bud group %s, offline, ", bud->id, bud->group );
		char id[ 256 ];
		strncpy( id, (const char*) bud->id, 255 );
		char gr[ 256 ];
		strncpy( gr, (const char*) bud->group, 255 );
		MessengerThread::_messenger_contacts->addContact( id, gr, "offline" );
	}
	qDebug( "ext_yahoo_got_buddies quit" );
}

//! Called when the ignore list is got from the server
void ext_yahoo_got_ignore( int id, YList* igns ) {
	qDebug( "ext_yahoo_got_ignore" );
}

//! Called when the contact list is got from the server
void ext_yahoo_got_identities( int id, YList* ids ) {
	qDebug( "ext_yahoo_got_ignore" );
}

//! Called when the cookie list is got from the server
void ext_yahoo_got_cookies( int id ) {
	qDebug( "ext_yahoo_got_cookies" );
}

//! Called when remote user's status changes.
void ext_yahoo_status_changed( int id, char *who, int stat, char *msg, int away ) {
	qDebug( "ext_yahoo_status_changed who:%s stat:%d msg:%s away:%d", who, stat, msg, away );
	MessengerThread::_messenger_contacts->updateContact( who, stat, msg );
}

//! Called when remote user sends you a message.
void ext_yahoo_got_im( int id, char *who, char *msg, long tm, int stat, int utf8 ) {
	qDebug( "ext_yahoo_got_im" );
	MessengerContacts::instance()->receivedMessage( who, msg );
}

//! Called when remote user sends you a conference invitation.
void ext_yahoo_got_conf_invite( int id, char *who, char *room, char *msg, YList *members ) {
}

//! Called when someone declines to join the conference.
void ext_yahoo_conf_userdecline( int id, char *who, char *room, char *msg ) {
}

//! Called when someone joins the conference.
void ext_yahoo_conf_userjoin( int id, char *who, char *room ) {
}

//! Called when someone leaves the conference.
void ext_yahoo_conf_userleave( int id, char *who, char *room ) {
}

//! Called when joining the chatroom.
void ext_yahoo_chat_cat_xml( int id, char *xml ) {
}

//! Called when joining the chatroom.
void ext_yahoo_chat_join( int id, char *room, char *topic, YList *members, int fd ) {
}

//! Called when someone joins the chatroom.
void ext_yahoo_chat_userjoin( int id, char *room, struct yahoo_chat_member *who ) {
}

//! Called when someone leaves the chatroom.
void ext_yahoo_chat_userleave( int id, char *room, char *who ) {
}

//! Called when someone messages in the chatroom.
void ext_yahoo_chat_message( int id, char *who, char *room, char *msg, int msgtype, int utf8 ) {
}

//! called when yahoo disconnects your chat session
void ext_yahoo_chat_yahoologout( int id ) {
}

//! called when yahoo sends back an error to you
void ext_yahoo_chat_yahooerror( int id ) {
}

//! Called when someone messages in the conference.
void ext_yahoo_conf_message( int id, char *who, char *room, char *msg, int utf8 ) {
}

//! Called when someone sends you a file
void ext_yahoo_got_file( int id, char *who, char *url, long expires, char *msg, char *fname, unsigned long fesize ) {
}

//! Called when a contact is added to your list
void ext_yahoo_contact_added( int id, char *myid, char *who, char *msg ) {
	qDebug( "A" );
}

//! Called when a contact rejects your add
void ext_yahoo_rejected( int id, char *who, char *msg ) {
}

//! Called when remote user starts or stops typing.
void ext_yahoo_typing_notify( int id, char *who, int stat ) {
	qDebug( "ext_yahoo_typing_notify" );
}

//! Called when remote user starts or stops a game.
void ext_yahoo_game_notify( int id, char *who, int stat ) {
}

//! Called when you receive mail, or with number of messages
void ext_yahoo_mail_notify( int id, char *from, char *subj, int cnt ) {
}

//! System message
void ext_yahoo_system_message( int id, char *msg ) {
	qDebug( "ext_yahoo_system_message %s", msg );
}

//! Called when you get a webcam update
void ext_yahoo_got_webcam_image( int id, const char * who, const unsigned char *image, unsigned int image_size, unsigned int real_size, unsigned int timestamp ) {
}

//! Called when you get a webcam invitation
void ext_yahoo_webcam_invite( int id, char *from ) {
}

//! Called when you get a response to a webcam invitation
void ext_yahoo_webcam_invite_reply( int id, char *from, int accept ) {
}

//! Called when the webcam connection closed
void ext_yahoo_webcam_closed( int id, char *who, int reason ) {
}

//! Called when the search result received from server
void ext_yahoo_got_search_result( int id, int found, int start, int total, YList *contacts ) {
	qDebug( "ext_yahoo_got_search_result" );
}

//! Called on error.
void ext_yahoo_error( int id, char *err, int fatal ) {
	qDebug( "ext_yahoo_error" );
}

//! Called when a viewer disconnects/connects/requests to connect
void ext_yahoo_webcam_viewer( int id, char *who, int connect ) {
}

//! Called when you get a request for webcam images
void ext_yahoo_webcam_data_request( int id, int send ) {
}

//! Called to log a message.
int ext_yahoo_log( char *fmt, ... ) {
	qDebug( "ext_yahoo_log" );
}

//! Add a listener for the fd.  Must call yahoo_read_ready
int ext_yahoo_add_handler( int id, int fd, yahoo_input_condition cond, void *data ) {
	qDebug( "ext_yahoo_add_handler" );
    struct _conn *c = y_new0(struct _conn, 1);
    c->tag = ++connection_tags;
    c->id = id;
    c->fd = fd;
    c->cond = cond;
    c->data = data;
    connections = y_list_prepend(connections, c);
    return c->tag;
}

//! Remove the listener for the fd.
void ext_yahoo_remove_handler( int id, int tag ) {
	qDebug( "ext_yahoo_remove_handler" );
    YList *l;
    for(l = connections; l; l = y_list_next(l)) {
        struct _conn *c = (struct _conn*) l->data;
        if(c->tag == tag) {
            /* don't actually remove it, just mark it for removal */
            /* we'll remove when we start the next poll cycle */
            c->remove = 1;
            return;
        }
    }
}

//! Connect to a host:port
int ext_yahoo_connect( char *host, int port ) {
	qDebug( "ext_yahoo_connect" );
    struct sockaddr_in serv_addr;
    static struct hostent *server;
    static char last_host[256];
    int servfd;
    char **p;
    if(last_host[0] || strcasecmp(last_host, host)!=0) {
        if(!(server = gethostbyname(host))) {
            return -1;
        }
        strncpy(last_host, host, 255);
    }
    if((servfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        return -1;
    }
    for (p = server->h_addr_list; *p; p++)
    {
        memset(&serv_addr, 0, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        memcpy(&serv_addr.sin_addr.s_addr, *p, server->h_length);
        serv_addr.sin_port = htons(port);
        if(connect(servfd, (struct sockaddr *) &serv_addr,
                    sizeof(serv_addr)) == -1) {
            if(errno!=ECONNREFUSED && errno!=ETIMEDOUT &&
                    errno!=ENETUNREACH) {
                break;
            }
        } else {
            return servfd;
        }
    }
    close(servfd);
    return -1;
}

//! Connect to a host:port asynchronously.
int ext_yahoo_connect_async( int id, char *host, int port, yahoo_connect_callback callback, void *data ) {
	qDebug( "ext_yahoo_connect_async" );
	sleep( 1 );
    struct sockaddr_in serv_addr;
    static struct hostent *server;
    int servfd;
    struct connect_callback_data * ccd;
    int error;
    if(!(server = gethostbyname( host ))) {
		qDebug( "ext_yahoo_connect_async error (0) %s", host );
        errno=h_errno;
        return -1;
    }
    if((servfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		qDebug( "ext_yahoo_connect_async error (1) %m" );
        return -1;
    }
    memset(&serv_addr, 0, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    memcpy(&serv_addr.sin_addr.s_addr, *server->h_addr_list, server->h_length);
    serv_addr.sin_port = htons(port);
    error = connect(servfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr));
    if(!error) {
		qDebug( "ext_yahoo_connect_async ok" );
        callback(servfd, 0, data);
        return 0;
    } else if(error == -1 && errno == EINPROGRESS) {
        ccd = (connect_callback_data*) calloc(1, sizeof(struct connect_callback_data));
        ccd->callback = callback;
        ccd->callback_data = data;
        ccd->id = id;
        ccd->tag = ext_yahoo_add_handler(-1, servfd, YAHOO_INPUT_WRITE, ccd);
		qDebug( "ext_yahoo_connect_async return" );
        return ccd->tag;
    } else {
		qDebug( "ext_yahoo_connect_async error (2) %m" );
        close(servfd);
        return -1;
    }
	qDebug( "ext_yahoo_connect_async end" );
}
