#include "QueryHitEntry.h"
#include "QtellaSub.h"
#include <qlistview.h>

const int QueryHitEntry::Busy = 1;
const int QueryHitEntry::Ready = 2;
const int QueryHitEntry::Firewalled = 4;
const int QueryHitEntry::Unknown = 8;

const int QueryHitEntry::GNUTELLA = 0;
const int QueryHitEntry::BITTORRENT = 1;
const int QueryHitEntry::BITTORRENT_FILE = 2;

QueryHitEntry::QueryHitEntry( const QString& filename, const Q_UINT32 size, const Q_UINT32 index, const Q_UINT32 speed, int status, Address& addr, std::string& id, QListViewItem *item, std::string& sha1 )
{
	_search_item_delete = true;
	networktype = GNUTELLA;
	_destination_file = filename;
	this->filename = filename.ascii();
	this->size = size;
	this->index = index;
	this->addr = addr;
	this->id = id;
	this->item = item;
	this->speed = speed;
	this->status = status;
	this->sha1 = sha1;
}

QueryHitEntry::QueryHitEntry( const QString& filename, const Q_UINT32 size, const QString& torrentsource, int network )
{
	_torrent_source = torrentsource;
	_search_item_delete = true;
	this->filename = filename.ascii();
	this->size = size;
	_destination_file = filename;
	index = 0;
	addr = Address();
	id = "";
	item = NULL;
	speed = 0;
	status = Unknown;
	sha1 = "";
	networktype = network;
}

QueryHitEntry::QueryHitEntry()
{
	_search_item_delete = true;
	networktype = -1;
	filename = "";
	size = 0;
	index = 0;
	addr = Address();
	id = "";
	item = NULL;
	speed = 0;
	status = Unknown;
}

QueryHitEntry::~QueryHitEntry()
{
	if( item && _search_item_delete ) {
		QtellaSub* p = QtellaSub::getInstance();
		p->_search_item_set.erase( p->_search_item_set.find( item ) );
		delete item;
	}
}

bool QueryHitEntry::firewalled()
{
	return ( ( status & Firewalled ) > 0 );
}

bool QueryHitEntry::equal( QueryHitEntry* q ) const
{
	return( ( filename == q->filename ) && ( size == q->size ) && ( index == q->index ) && ( _torrent_source == q->_torrent_source ) );
}

int QueryHitEntry::getNetworkType()
{
	return networktype;
}

void QueryHitEntry::setDestinationFilename( const QString& n )
{
	_destination_file = n;
}

void QueryHitEntry::setDestinationDirectory( const QString& n )
{
	_destination_dir = n;
}

const QString& QueryHitEntry::getDestinationDirectory() const
{
	return _destination_dir;
}

const QString& QueryHitEntry::getDestinationFilename() const
{
	return _destination_file;
}

void QueryHitEntry::setSearchItem( QListViewItem* item, bool d )
{
	this->item = item;
	_search_item_delete = d;
}

const QString& QueryHitEntry::getTorrentSource() const
{
	return _torrent_source;
}

void QueryHitEntry::setTorrentSource( const QString& srcurl )
{
	_torrent_source = srcurl;
}

